/* cth3.C - A program to generate Feynman graphs In Toda Field Theory
   Copyright (C) Paul Bayton 1994
      This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     as published by the Free Software Foundation; either version 2
     of the License, or (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


// program ctg1.C
// object: calculation of feynman diograms in toda theory

#include <iostream.h>
#include <stdio.h>

#define GROUPE7
#define ORDER 2

// set external mass and angles
int em1=4;
int ea1=36;
int em2=4;
int ea2=0;


// *************************************************** 
int dino2=1;

#ifdef GROUPD6
const cox=10;				// coxeter number
const Mmasses=6;
const int PI=60;
#endif
#ifdef GROUPE6
const cox=12;
const Mmasses=6;
const PI=12;
#endif
#ifdef GROUPE7
const cox=18;
const Mmasses=7;
const PI=18;
#endif
#ifdef GROUPE8
const cox=30;
const Mmasses=8;
const PI=30;
#endif

#if ORDER==2
 const Mnodes=4, Mlinks=4;		// second order
 char* nodestr="nodelst={0,1,2,3}";
#endif
#if ORDER==3
 const Mnodes=6, Mlinks=7; 		// third order
 char* nodestr="nodelst={0,1,2,3,4,5}";
#endif
#if ORDER==4
 const Mnodes=8, Mlinks=10; 		// fourth order
 char* nodestr="nodelst={0,1,2,3,4,5,6,7}";
#endif
#if ORDER==5
 const Mnodes=10, Mlinks=13;		// fith order
 char* nodestr="nodelst={0,1,2,3,4,5,6,7,8,9}";
#endif

class Angle
{
public:
int t;
enum { TwoPi=2*PI, Pi=PI };			// defines radix
Angle(int i=0) { t=(i+TwoPi) % TwoPi; };
operator int() { return (TwoPi+this->t) % TwoPi; };		// convert angle to int
//Angle operator=(Angle a) { t=(a.t+TwoPi) % TwoPi; return *this; };
int operator==(int i) { return t==((i+TwoPi) % TwoPi); };
//Angle operator+(Angle a) { Angle b; b.t=(t+a.t) % TwoPi; return b; };
//Angle operator-(Angle a) { Angle b; b.t=(TwoPi+t-a.t) % TwoPi; return b; };
};

// change also Mass::prep external masses and Theta::prep external angles
// for second order, change opto=4 to upto=3 in Dio::make (186)
// change also p("nodelst=..... for particular order.

int pass;

class Dio {
public:
static int used[Mlinks];
static Dio *pd,*pdstart,*pdend;
int *start,*end,upto;
int search1(int i);	// search used for non-zero starting at i
int update(),rest();		// make another to link 
static int bd,before[5000][Mlinks+1][2];
static permlst[25][Mnodes+1];
static int init;
static int list[Mlinks+1][2];	// main linklist
int Dio::check();
void Dio::gen_permed_dios();
void gen_perms();
int make();		// main program-needs first link only
void prep(Dio* d);
};


int Dio::check() { 
// check weather or not the current dio has been produced before with permutated nodes
// 1:true=>ok (not been before) 0=>fail (before)
int i,j,ok2;
for(i=0;i<bd;i++) {
ok2=1;
for(j=0;j<Mlinks && ok2;j++) {
 if(list[j][0]!=before[i][j][0]) ok2=0;
 if(list[j][1]!=before[i][j][1]) ok2=0;
 };
if(ok2) return 0;
};
return 1;
};



void Dio::gen_permed_dios() { // generate a list of dios which are permuted from current list
int i,j,k,n,p,p3;
int list2[Mlinks+1][2],list3[Mlinks+1][2];
int used[Mlinks+1]; // once a link is used append to list3 and dont use again
int plist[Mlinks+1][2]; // partial list with plist[i][0]=node in common
int gooddio,t0,t1,u;
Dio tmp;
// go through node permutation list permuting list and storing in before
int mperm=24;
for(i=0;i<mperm;i++) {
gooddio=1;
p3=0;
 for(j=0;j<Mlinks;j++) {
  list2[j][0]=permlst[i][ list[j][0] ];
  list2[j][1]=permlst[i][ list[j][1] ];
  }
 for(j=0;j<Mlinks;j++) used[j]=1;
 for(n=0;n<Mnodes;n++) {

 // find set of links corresponding to node n
 p=0;
 for(j=0;j<Mlinks;j++) {
  if((n==list2[j][0] || n==list2[j][1]) && used[j]) { 
    plist[p][0]=n;
    if(n==list2[j][0]) plist[p][1]=list2[j][1]; else plist[p][1]=list2[j][0];
    used[j]=0;
    p++; }
  }
 // now sort as (n,p)(n,q) with p<q
 for(j=0;j<p;j++) for(k=j;k<p;k++) 
 if(plist[j][1] > plist[k][1]) {
  t0=plist[j][0]; t1=plist[j][1];
  plist[j][0]=plist[k][0]; plist[j][1]=plist[k][1];
  plist[k][0]=t0; plist[k][1]=t1;
  }

 // append links to list3- a sorted version of list2 with node first.
 for(j=0;j<p;j++) {
  list3[p3][0]=plist[j][0];
  list3[p3][1]=plist[j][1];
  p3++;
  };
                  // cout<<"n is"<<n<<",p is"<<p<<",p3 is"<<p3<<"\n";
 }; // end of for n

 // check list3 for i>j in (i,j) and u order
 u=4; gooddio=1;
 for(j=0;j<Mlinks;j++) {
  if(list3[j][0]>list3[j][1]) gooddio=0;
  if(list3[j][1]==u+1) u++;
  if(list3[j][1]>u) gooddio=0;
  };

 // store dio on before
 if(gooddio) {
  for(j=0;j<Mlinks;j++) {
   before[bd][j][0]=list3[j][0];
   before[bd][j][1]=list3[j][1];
   };
  bd++;
  };
                      /* cout<<gooddio<<flush;

cout << "linklst={{ef0,0},{ef1,1},{e2,2},{e3,3},";
for(j=0;j<Mlinks;j++) {
cout << "{"<< list3[j][0] << "," << list3[j][1] << "}";
if(j<Mlinks-1) cout << ",";
};
cout << "}\n"<<flush;     */


 }; // end of for i (permutations)
};


void Dio::gen_perms() {
// generate perm list permute nodes 4,5,6..Mnodes-1 not 0,1,2,3
int i,j,pl,p,q,m;
pl=0;
for(i=0;i<Mnodes;i++) permlst[pl][i]=i;
for(;;) {
for(i=0;i<Mnodes;i++) permlst[pl+1][i]=permlst[pl][i];
pl++;
p=Mnodes-1;
do {
p--;
do {
permlst[pl][p]++;
m=0;
for(j=4;j<p;j++) if(permlst[pl][j]==permlst[pl][p]) m=1;
} while(m && permlst[pl][p]<Mnodes);
} while(permlst[pl][p]>=Mnodes && p>=4);
if(p<4) return;
p++;
for(q=p;q<Mnodes;q++) {
permlst[pl][q]=3;
do {
permlst[pl][q]++;
m=0;
for(j=4;j<q;j++) if(permlst[pl][j]==permlst[pl][q]) m=1;
} while(m);
};
};
};



struct Massdat {public: int l1,l2,l3,ninst,node,depth; };

class Mass:public Massdat
{
private:
enum { Mp1d=64, Mp2d=10 }; // *implimentation limit, ok for e8
static int c[Mmasses+1][Mmasses+1][Mmasses+1];
static struct S {int first,second; } p1[Mmasses+1][Mp1d];
static int p2[Mmasses+1][Mmasses+1][Mp2d];
static int p1max[Mmasses+1],p2max[Mmasses+1][Mmasses+1];
void gen();
void gen_p();
int update(),rest();
public:
enum { Mpd=50 }; // *impimentation limit max 2*propagators
static Mass *pdstart,*pdend,*pd;
int *m1,*m2,*m3;
static int init;
Mass() {gen(); gen_p(); };
void debug();
int make();
void prep(Dio& d,Mass*m);
static int list[Mlinks+4];
};


class Theta:public Massdat
{
public:
Angle t1,t2,t3;
int io1,io2,io3;
static Theta *pd,*pdstart,*pdend;
static int store[500][4],sd; // *implimentation limit, max number of couplings
static int th[Mmasses][Mmasses][Mmasses];
int update(),rest();
void set(int,int,int,int),gen();
public:
static int init;
int make();
static int list[Mlinks+4];
void prep(Dio& d,Mass*m,Theta*t);
Theta() { gen(); };
};

// *********************************************** end of headers

// overload p;		// print functions
void p(int i) { cout<<i<<'\n'<<flush; };
void p(char* c) { cout<<c<<'\n'<<flush; };
void p(int*x,int n) {
cout<<"list :";
for(int i=0;i<n;i++) cout<<*(x+i)<<",";
cout<<".\n"<<flush;
};


ostream& operator<<(ostream& s,Dio&d) {
s << "linklst={{ef0,0},{ef1,1},{ef2,2},{ef3,3},";
for(int i=0;i<Mlinks;i++) {
s << "{"<< d.list[i][0] << "," << d.list[i][1] << "}";
if(i<Mlinks-1) s << ",";
};
s << "}\n"<<flush;
return s;
};

#ifdef GROUPD6
ostream& operator<<(ostream& s,Mass&m) {
char*s1="s1",*s2="s2";
s << "masslst={ "<<m.list[Mlinks]<<","<<m.list[Mlinks+1]<<
","<<m.list[Mlinks+2]<<","<<m.list[Mlinks+3]<<",";
for(int i=0;i<Mlinks;i++) {
if(m.list[i]==Mmasses) s<<"s2"; else if(m.list[i]==Mmasses-1) s<<"s1"; else s<<m.list[i];
if(i<Mlinks-1) s << ",";
};
s << "}\n"<<flush;
return s;
};
#endif

#ifdef GROUPE6
ostream& operator<<(ostream& s,Mass&m) {
s << "masslst={ "<<m.list[Mlinks]<<","<<m.list[Mlinks+1]<<
","<<m.list[Mlinks+2]<<","<<m.list[Mlinks+3]<<",";
for(int i=0;i<Mlinks;i++) {
s<<m.list[i];
if(i<Mlinks-1) s << ",";
};
s << "}\n"<<flush;
return s;
};
#endif

#ifdef GROUPE7
ostream& operator<<(ostream& s,Mass&m) {
s << "masslst={ "<<m.list[Mlinks]<<","<<m.list[Mlinks+1]<<
","<<m.list[Mlinks+2]<<","<<m.list[Mlinks+3]<<",";
for(int i=0;i<Mlinks;i++) {
s<<m.list[i];
if(i<Mlinks-1) s << ",";
};
s << "}\n"<<flush;
return s;
};
#endif

#ifdef GROUPE8
ostream& operator<<(ostream& s,Mass&m) {
s << "masslst={ "<<m.list[Mlinks]<<","<<m.list[Mlinks+1]<<
","<<m.list[Mlinks+2]<<","<<m.list[Mlinks+3]<<",";
for(int i=0;i<Mlinks;i++) {
s<<m.list[i];
if(i<Mlinks-1) s << ",";
};
s << "}\n"<<flush;
return s;
};
#endif


ostream& operator<<(ostream& s,Theta&t) {
s << "thetalst={ "<<t.list[Mlinks]<<","<<t.list[Mlinks+1]<<
","<<t.list[Mlinks+2]<<","<<t.list[Mlinks+3]<<",";
for(int i=0;i<Mlinks;i++) {
s<<t.list[i];
if(i<Mlinks-1) s << ",";
};
s << "}\n"<<flush;
return s;
};

/*
ostream& operator<<(ostream& s,Theta&t) {
s << "thetalst={ "<< ((2*PI)-t.list[Mlinks]) % (2*PI)<<","<< ((2*PI)-t.list[Mlinks+1]) % (2*PI)<<
","<< ((2*PI)-t.list[Mlinks+2]) % (2*PI)<<","<<  ((2*PI)-t.list[Mlinks+3]) % (2*PI)<<",";
for(int i=0;i<Mlinks;i++) {
s<< ((2*PI)-t.list[i]) % (2*PI);
if(i<Mlinks-1) s << ",";
};
s << "}\n"<<flush;
return s;
};
*/

// ********************************************* end of print functions


inline int Dio::search1(int i) 
 { while((!pd->used[i]) && (i < Mnodes)) i++; return i; }

void Dio::prep(Dio* d) {
for(int i=0;i<Mlinks+2;i++) {
pd=d+i;
pd->start=&list[i][0];
pd->end=&list[i][1];
*pd->start=*pd->end=0;
used[i]=3;
};
pdend=d+(Mlinks-1);
pdstart=d;
init=1;
};

int Dio::update()
{
// start is the same; end needs updateing
// p("debug:update"); cout<<int(pd-pdstart)<<*start<<*end<<upto<<":";
// cout<<used[0]<<used[1]<<used[2]<<used[3]<<used[4]<<used[5]; p("uused");
*end=search1(*end+1);
if(*end>upto) {
if(this==pdstart) { return 2; } 
else { pd--; used[*pd->start]++; used[*pd->end]++; return 1; } };
if(pd>=pdend) { p("debug:1"); return 0; }
pd++;				// setup used,upto,start for next call
used[*start]--; used[*end]--;
if(*end==upto && upto<(Mnodes-1)) pd->upto=upto+1; else pd->upto=upto;
if(used[*start]) { *pd->start=*start; *pd->end=search1(*end+1); }
else { *pd->start=search1(*start); *pd->end=search1(*pd->start+1); };
int i=pd->rest();
return i;
};

int Dio::rest()
{ // p("debug:rest"); cout<<int(pd-pdstart)<<*start<<*end<<upto<<":";
// cout<<used[0]<<used[1]<<used[2]<<used[3]<<used[4]<<used[5]; p("rused");
if(*end>upto) {
if(this==pdstart) { return 2; } 
else { pd--; used[*pd->start]++; used[*pd->end]++; return 1; } };
if(pd>=pdend) { return 0; }
pd++;				// setup used,upto,start for next call
used[*start]--; used[*end]--;
if(*end==upto && upto<(Mnodes-1)) pd->upto=upto+1; else pd->upto=upto;
if(used[*start]) { *pd->start=*start; *pd->end=search1(*end+1); }
else { *pd->start=search1(*start); *pd->end=search1(*pd->start+1); };
int i=pd->rest();
return i;
};

int Dio::make()
{
int l;
if(init) { init=0; pd=pdstart; *pd->start=0; *pd->end=1; 
#if ORDER==2
pd->upto=3; 	// >>>upto=3/4
#elif ORDER==3
pd->upto=4;
#elif ORDER==4
pd->upto=4;
#elif ORDER==5
pd->upto=4;
#endif

l=pd->rest();
if(l==0) return 1;
if(l==2) return 0;
};
for(;;) {
l=pd->update();
if(l==0) return 1;
if(l==2) return 0;
};
return 0;
};


// *********************************************** end of Dio functions

#ifdef GROUPD6
void Mass::gen()
{
int i,j,k;
int s1=Mmasses-1, s2=Mmasses;
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++) c[i][j][k]=0;
		// d_n n=even:
for(i=1;i<=Mmasses-2;i++)
 for(j=1;j<=Mmasses-2;j++)
  for(k=1;k<=Mmasses-2;k++) {
if(i+j-k==0) c[i][j][k]=1; 
if(i-j+k==0) c[i][j][k]=1;
if(i-j-k==0) c[i][j][k]=1;
if(i+j+k==cox) c[i][j][k]=1;
   };
for(i=2;i<=Mmasses-2;i++,i++) {
c[i][s1][s1]=1; c[i][s2][s2]=1;
c[s1][i][s1]=1; c[s2][i][s2]=1;
c[s1][s1][i]=1; c[s2][s2][i]=1;
 };
for(i=1;i<=Mmasses-2;i++,i++) {
c[i][s1][s2]=1; c[i][s2][s1]=1;
c[s1][i][s2]=1; c[s2][i][s1]=1;
c[s1][s2][i]=1; c[s2][s1][i]=1;
}; 
// symerterise:
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++)
   if(c[i][j][k]) c[i][j][k]=c[i][k][j]=c[j][i][k]=c[j][k][i]=c[k][i][j]=c[k][j][i]=1;
}; 		// of Mass::gen
#endif

#ifdef GROUPE6
void Mass::gen()
{
int i,j,k;
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++) c[i][j][k]=0;

c[1][1][2]=1;
c[1][1][4]=1;
c[1][2][3]=1;
c[1][3][1]=1;
c[1][3][4]=1;
c[1][4][2]=1;
c[1][4][5]=1;
c[1][5][3]=1;
c[1][5][6]=1;
c[1][6][4]=1;
c[2][2][1]=1;
c[2][2][4]=1;
c[2][3][2]=1;
c[2][3][5]=1;
c[2][4][3]=1;
c[2][4][6]=1;
c[2][5][1]=1;
c[2][5][4]=1;
c[2][6][5]=1;
c[3][3][3]=1;
c[3][3][6]=1;
c[3][4][1]=1;
c[3][5][2]=1;
c[3][6][3]=1;
c[3][6][6]=1;
c[4][4][2]=1;
c[4][4][5]=1;
c[4][5][6]=1;
c[4][6][1]=1;
c[4][6][4]=1;
c[5][5][1]=1;
c[5][5][4]=1;
c[5][6][2]=1;
c[5][6][5]=1;
c[6][6][3]=1;
c[6][6][6]=1;

// symerterise:
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++)
   if(c[i][j][k]) c[i][j][k]=c[i][k][j]=c[j][i][k]=c[j][k][i]=c[k][i][j]=c[k][j][i]=1;
};
#endif

#ifdef GROUPE7
void Mass::gen()
{
int i,j,k;
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++) c[i][j][k]=0;

c[1][1][2]=1;
c[1][1][4]=1;
c[1][2][1]=1;
c[1][2][3]=1;
c[1][3][2]=1;
c[1][3][4]=1;
c[1][3][5]=1;
c[1][4][1]=1;
c[1][4][3]=1;
c[1][4][6]=1;
c[1][5][3]=1;
c[1][5][6]=1;
c[1][6][4]=1;
c[1][6][5]=1;
c[1][6][7]=1;
c[1][7][6]=1;
c[2][2][2]=1;
c[2][2][4]=1;
c[2][2][5]=1;
c[2][3][1]=1;
c[2][3][3]=1;
c[2][3][6]=1;
c[2][4][2]=1;
c[2][4][5]=1;
c[2][5][2]=1;
c[2][5][4]=1;
c[2][5][7]=1;
c[2][6][3]=1;
c[2][7][5]=1;
c[2][7][7]=1;
c[3][3][2]=1;
c[3][3][7]=1;
c[3][4][1]=1;
c[3][5][1]=1;
c[3][5][6]=1;
c[3][6][2]=1;
c[3][6][5]=1;
c[3][6][7]=1;
c[3][7][3]=1;
c[3][7][6]=1;
c[4][4][4]=1;
c[4][4][5]=1;
c[4][4][7]=1;
c[4][5][2]=1;
c[4][5][4]=1;
c[4][5][7]=1;
c[4][6][1]=1;
c[4][6][6]=1;
c[4][7][4]=1;
c[4][7][5]=1;
c[5][5][5]=1;
c[5][6][1]=1;
c[5][6][3]=1;
c[5][7][2]=1;
c[5][7][4]=1;
c[5][7][7]=1;
c[6][6][4]=1;
c[6][6][7]=1;
c[6][7][1]=1;
c[6][7][3]=1;
c[6][7][6]=1;
c[7][7][2]=1;
c[7][7][5]=1;
c[7][7][7]=1;



// symerterise:
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++)
   if(c[i][j][k]) c[i][j][k]=c[i][k][j]=c[j][i][k]=c[j][k][i]=c[k][i][j]=c[k][j][i]=1;
};
#endif

#ifdef GROUPE8
void Mass::gen()
{
int i,j,k;
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++) c[i][j][k]=0;



c[1][1][1]=1;
c[1][1][2]=1;
c[1][1][3]=1;
c[1][2][1]=1;
c[1][2][2]=1;
c[1][2][3]=1;
c[1][2][4]=1;
c[1][3][1]=1;
c[1][3][2]=1;
c[1][3][4]=1;
c[1][3][5]=1;
c[1][4][2]=1;
c[1][4][3]=1;
c[1][4][4]=1;
c[1][4][5]=1;
c[1][4][6]=1;
c[1][5][3]=1;
c[1][5][4]=1;
c[1][5][6]=1;
c[1][5][7]=1;
c[1][6][4]=1;
c[1][6][5]=1;
c[1][6][7]=1;
c[1][7][5]=1;
c[1][7][6]=1;
c[1][7][8]=1;
c[1][8][7]=1;
c[1][8][8]=1;
c[2][2][1]=1;
c[2][2][2]=1;
c[2][2][4]=1;
c[2][2][5]=1;
c[2][2][6]=1;
c[2][3][1]=1;
c[2][3][3]=1;
c[2][3][6]=1;
c[2][4][1]=1;
c[2][4][2]=1;
c[2][4][7]=1;
c[2][5][2]=1;
c[2][5][6]=1;
c[2][6][2]=1;
c[2][6][3]=1;
c[2][6][5]=1;
c[2][6][7]=1;
c[2][6][8]=1;
c[2][7][4]=1;
c[2][7][6]=1;
c[2][7][7]=1;
c[2][7][8]=1;
c[2][8][6]=1;
c[2][8][7]=1;
c[3][3][2]=1;
c[3][3][3]=1;
c[3][3][5]=1;
c[3][3][6]=1;
c[3][3][7]=1;
c[3][4][1]=1;
c[3][4][5]=1;
c[3][5][1]=1;
c[3][5][3]=1;
c[3][5][4]=1;
c[3][5][7]=1;
c[3][5][8]=1;
c[3][6][2]=1;
c[3][6][3]=1;
c[3][6][6]=1;
c[3][6][8]=1;
c[3][7][3]=1;
c[3][7][5]=1;
c[3][8][5]=1;
c[3][8][6]=1;
c[3][8][8]=1;
c[4][4][1]=1;
c[4][4][4]=1;
c[4][4][6]=1;
c[4][4][7]=1;
c[4][4][8]=1;
c[4][5][1]=1;
c[4][5][3]=1;
c[4][5][5]=1;
c[4][5][8]=1;
c[4][6][1]=1;
c[4][6][4]=1;
c[4][7][2]=1;
c[4][7][4]=1;
c[4][7][7]=1;
c[4][7][8]=1;
c[4][8][4]=1;
c[4][8][5]=1;
c[4][8][7]=1;
c[5][5][4]=1;
c[5][5][5]=1;
c[5][5][8]=1;
c[5][6][1]=1;
c[5][6][2]=1;
c[5][6][7]=1;
c[5][7][1]=1;
c[5][7][3]=1;
c[5][7][6]=1;
c[5][8][3]=1;
c[5][8][4]=1;
c[5][8][5]=1;
c[5][8][8]=1;
c[6][6][3]=1;
c[6][6][6]=1;
c[6][6][8]=1;
c[6][7][1]=1;
c[6][7][2]=1;
c[6][7][5]=1;
c[6][7][8]=1;
c[6][8][2]=1;
c[6][8][3]=1;
c[6][8][6]=1;
c[6][8][7]=1;
c[7][7][2]=1;
c[7][7][4]=1;
c[7][7][7]=1;
c[7][8][1]=1;
c[7][8][2]=1;
c[7][8][4]=1;
c[7][8][6]=1;
c[7][8][8]=1;
c[8][8][1]=1;
c[8][8][3]=1;
c[8][8][5]=1;
c[8][8][7]=1;
c[8][8][8]=1;


// symerterise:
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++) 
   if(c[i][j][k]) c[i][j][k]=c[i][k][j]=c[j][i][k]=c[j][k][i]=c[k][i][j]=c[k][j][i]=1;
};
#endif


void Mass::gen_p()
{
int i,j,k,n;
for(i=1;i<=Mmasses;i++) {
n=0;
 for(j=1;j<=Mmasses;j++)
  for(k=1;k<=Mmasses;k++)
   if(c[i][j][k]) { p1[i][n].first=j; p1[i][n++].second=k; };
p1max[i]=n;
};
for(i=1;i<=Mmasses;i++)
 for(j=1;j<=Mmasses;j++) {
  n=0;
  for(k=1;k<=Mmasses;k++) if(c[i][j][k]) p2[i][j][n++]=k;
p2max[i][j]=n;
};
}; 		// of Mass::gen_p

void Mass::debug()
{
for(int i=1;i<=Mmasses;i++)
 for(int j=1;j<=Mmasses;j++)
  for(int k=1;k<=Mmasses;k++) 
if(c[i][j][k]) cout << i << j << k << "\n";
};

// ******************************************* end of mass setup functions

void Mass::prep(Dio& d,Mass* m) {
pd=pdstart=m;
int i,j,k,ni;
int link[Mnodes][3];
int inst[Mlinks+4];		// dummy links Mlinks..Mlinks+3
int nused[Mnodes];		// nodes used
// 1. set inst
for(i=0;i<Mlinks;i++) inst[i]=0;
inst[Mlinks]=inst[Mlinks+1]=inst[Mlinks+2]=inst[Mlinks+3]=1;
// 1b. set mass table
for(i=0;i<Mlinks+4;i++) list[i]=0;
list[Mlinks]=list[Mlinks+2]=em1;
list[Mlinks+1]=list[Mlinks+3]=em2;
// 2. generate links per node list
for(i=0;i<Mnodes;i++) {
nused[i]=0;
k=0;
if(pass==1) if(i<4) link[i][k++]=Mlinks+i;
if(pass>1) { 
	if(i==0) { link[i][k++]=Mlinks; link[i][k++]=Mlinks+1; }
	if(i==1) link[i][k++]=Mlinks+2;
	if(i==2) link[i][k++]=Mlinks+3;
	}
for(j=0;j<Mlinks;j++) {
if(d.list[j][0]==i || d.list[j][1]==i) link[i][k++]=j; } 
};
// 3. determin order of evaluation and set arguments
do {
// 3a. determin node to use next
ninst=0; node=-1;
for(i=0;i<Mnodes;i++) {
ni=0;
if(inst[link[i][0]]) ni++; if(inst[link[i][1]]) ni++; if(inst[link[i][2]]) ni++;
if(ni>ninst && ni<3) { node=i; ninst=ni; };
};
if(node==-1) { i=0; while(nused[i]) i++; node=i; ninst=3; }		// check only now
// 3b. find out which links are instantated and which are not
k=0;  l1=link[node][0]; l2=link[node][1]; l3=link[node][2];
if(!inst[l1] && inst[l2]) { k=l1; l1=l2; l2=k; }	// mini sort of l1..l3
if(!inst[l1] && inst[l3]) { k=l1; l1=l3; l3=k; }
if(!inst[l2] && inst[l3]) { k=l2; l2=l3; l3=k; }
// 3c. set l1,l2,l3, ninst,node in ma
pd->l1=l1;
pd->l2=l2;
pd->l3=l3;
pd->node=node;
pd->ninst=ninst;
pd->m1=&list[l1];
pd->m2=&list[l2];
pd->m3=&list[l3];
// 3d. update inst
inst[link[node][0]]++; inst[link[node][1]]++; inst[link[node][2]]++; 
nused[node]=1;
pd++;
j=1; for(i=0;i<Mnodes;i++) { j=j && nused[i]; } // terminating condition: all nodes are used up
} while(!j);
init=1;
pdend=pd-1;
};

int Mass::update()
{ // p("debug::update"); cout<<depth<<int(pd-pdstart)<<*m1<<*m2<<*m3; p("d,pd,m1,m2,m3");
depth++;
switch(ninst) { 
case 1:
if(depth>=p1max[*m1]) { if(pd==pdstart) return 2; else { pd--; return 1; } };
*m2=p1[*m1][depth].first;
*m3=p1[*m1][depth].second;
break;
case 2:
if(depth>=p2max[*m1][*m2]) { if(pd==pdstart) return 2; else { pd--; return 1; } };
*m3=p2[*m1][*m2][depth];
break;
case 3:
if(pd==pdstart) return 2; else { pd--; return 1; }
};
if(pd>=pdend) return 0;
pd++;
return pd->rest();
};

int Mass::rest()
{ // p("debug::rest"); cout<<depth<<int(pd-pdstart)<<*m1<<*m2<<*m3; p("d,pd,m1,m2,m3");
depth=0;
switch(ninst) { 
case 1:
if(depth>=p1max[*m1]) { if(pd==pdstart) return 2; else { pd--; return 1; } };
*m2=p1[*m1][depth].first;
*m3=p1[*m1][depth].second;
break;
case 2:
if(depth>=p2max[*m1][*m2]) { if(pd==pdstart) return 2; else { pd--; return 1; } };
*m3=p2[*m1][*m2][depth];
break;
case 3:
if(!c[*m1][*m2][*m3]) { if(pd==pdstart) return 2; else { pd--; return 1; } };
};
if(this>=pdend) return 0;
pd++;
return pd->rest();
};

int Mass::make() {
int l;
if(init) { init=0; pd=pdstart; l=pd->rest(); // p(l); 
if(l==0) return 1;
if(l==2) return 0;
}
for(;;) {
l=pd->update();
if(l==0) return 1;
if(l==2) return 0;
};
return 0;
};


// *************************************************** end of theta functions
Dio d; Mass m; Theta t;

void Theta::set(int i,int j,int k,int l) {
store[sd][0]=i; store[sd][1]=j;		// store is temporary store for data
store[sd][2]=k; store[sd][3]=l;
sd++; };

#ifdef GROUPD6
void Theta::gen() {
// data taken from prolog
sd=0;
set(1,1,2,48);
set(1,2,1,6);
set(1,2,3,42);
set(1,3,2,12);
set(1,3,4,36);
set(1,4,3,18);
set(1,5,6,24);
set(1,6,5,24);
set(2,2,4,36);
set(2,3,1,6);
set(2,4,2,12);
set(2,4,4,24);
set(2,5,5,18);
set(2,6,6,18);
set(3,3,4,24);
set(3,4,1,6);
set(3,4,3,18);
set(3,5,6,12);
set(3,6,5,12);
set(4,4,2,12);
set(4,5,5,6);
set(4,6,6,6);
set(5,5,2,24);
set(5,5,4,48);
set(5,6,1,12);
set(5,6,3,36);
set(6,6,2,24);
set(6,6,4,48);
// make p1 list
for(int i=0;i<sd;i++) {
 th[store[i][0]][store[i][1]][store[i][2]]=store[i][3];		//p1[m1,m2,m3] is angle data
 th[store[i][1]][store[i][0]][store[i][2]]=store[i][3];
};
};
#endif

#ifdef GROUPE6
void Theta::gen() {
int i,j,k;
sd=0;


set(1,1,2,4);
set(1,1,4,10);
set(1,2,3,6);
set(1,3,1,3);
set(1,3,4,7);
set(1,4,2,1);
set(1,4,5,5);
set(1,5,3,3);
set(1,5,6,9);
set(1,6,4,2);
set(2,2,1,4);
set(2,2,4,10);
set(2,3,2,3);
set(2,3,5,7);
set(2,4,3,3);
set(2,4,6,9);
set(2,5,1,1);
set(2,5,4,5);
set(2,6,5,2);
set(3,3,3,4);
set(3,3,6,10);
set(3,4,1,2);
set(3,5,2,2);
set(3,6,3,1);
set(3,6,6,5);
set(4,4,2,2);
set(4,4,5,4);
set(4,5,6,6);
set(4,6,1,1);
set(4,6,4,3);
set(5,5,1,2);
set(5,5,4,4);
set(5,6,2,1);
set(5,6,5,3);
set(6,6,3,2);
set(6,6,6,4);


for(i=0;i<sd;i++) {
 th[store[i][0]][store[i][1]][store[i][2]]=store[i][3];
 th[store[i][1]][store[i][0]][store[i][2]]=store[i][3];
}
};
#endif


#ifdef GROUPE7
void Theta::gen() {
int i,j,k;
sd=0;

set(1,1,2,8);
set(1,1,4,16);
set(1,2,1,5);
set(1,2,3,11);
set(1,3,2,4);
set(1,3,4,8);
set(1,3,5,12);
set(1,4,1,1);
set(1,4,3,7);
set(1,4,6,15);
set(1,5,3,4);
set(1,5,6,10);
set(1,6,4,2);
set(1,6,5,6);
set(1,6,7,14);
set(1,7,6,3);
set(2,2,2,6);
set(2,2,4,10);
set(2,2,5,16);
set(2,3,1,3);
set(2,3,3,7);
set(2,3,6,13);
set(2,4,2,4);
set(2,4,5,10);
set(2,5,2,1);
set(2,5,4,5);
set(2,5,7,15);
set(2,6,3,3);
set(2,7,5,2);
set(2,7,7,8);
set(3,3,2,4);
set(3,3,7,16);
set(3,4,1,3);
set(3,5,1,2);
set(3,5,6,8);
set(3,6,2,2);
set(3,6,5,6);
set(3,6,7,10);
set(3,7,3,1);
set(3,7,6,5);
set(4,4,4,6);
set(4,4,5,8);
set(4,4,7,14);
set(4,5,2,3);
set(4,5,4,5);
set(4,5,7,11);
set(4,6,1,1);
set(4,6,6,7);
set(4,7,4,2);
set(4,7,5,4);
set(5,5,5,6);
set(5,6,1,2);
set(5,6,3,4);
set(5,7,2,1);
set(5,7,4,3);
set(5,7,7,7);
set(6,6,4,4);
set(6,6,7,8);
set(6,7,1,1);
set(6,7,3,3);
set(6,7,6,5);
set(7,7,2,2);
set(7,7,5,4);
set(7,7,7,6);



for(i=0;i<sd;i++) { 
 th[store[i][0]][store[i][1]][store[i][2]]=store[i][3];
 th[store[i][1]][store[i][0]][store[i][2]]=store[i][3];
}
};
#endif

#ifdef GROUPE8
void Theta::gen() {
int i,j,k;
sd=0;

set(1,1,1,10);
set(1,1,2,18);
set(1,1,3,28);
set(1,2,1,6);
set(1,2,2,12);
set(1,2,3,16);
set(1,2,4,22);
set(1,3,1,1);
set(1,3,2,9);
set(1,3,4,17);
set(1,3,5,27);
set(1,4,2,5);
set(1,4,3,9);
set(1,4,4,13);
set(1,4,5,19);
set(1,4,6,23);
set(1,5,3,2);
set(1,5,4,8);
set(1,5,6,16);
set(1,5,7,26);
set(1,6,4,5);
set(1,6,5,11);
set(1,6,7,21);
set(1,7,5,3);
set(1,7,6,7);
set(1,7,8,25);
set(1,8,7,4);
set(1,8,8,14);
set(2,2,1,6);
set(2,2,2,10);
set(2,2,4,16);
set(2,2,5,22);
set(2,2,6,28);
set(2,3,1,5);
set(2,3,3,11);
set(2,3,6,21);
set(2,4,1,3);
set(2,4,2,7);
set(2,4,7,25);
set(2,5,2,4);
set(2,5,6,14);
set(2,6,2,1);
set(2,6,3,5);
set(2,6,5,11);
set(2,6,7,17);
set(2,6,8,27);
set(2,7,4,3);
set(2,7,6,9);
set(2,7,7,13);
set(2,7,8,19);
set(2,8,6,2);
set(2,8,7,8);
set(3,3,2,8);
set(3,3,3,10);
set(3,3,5,16);
set(3,3,6,18);
set(3,3,7,26);
set(3,4,1,4);
set(3,4,5,14);
set(3,5,1,1);
set(3,5,3,7);
set(3,5,4,9);
set(3,5,7,17);
set(3,5,8,25);
set(3,6,2,4);
set(3,6,3,6);
set(3,6,6,12);
set(3,6,8,22);
set(3,7,3,2);
set(3,7,5,8);
set(3,8,5,3);
set(3,8,6,5);
set(3,8,8,13);
set(4,4,1,4);
set(4,4,4,10);
set(4,4,6,14);
set(4,4,7,18);
set(4,4,8,28);
set(4,5,1,3);
set(4,5,3,7);
set(4,5,5,11);
set(4,5,8,21);
set(4,6,1,2);
set(4,6,4,8);
set(4,7,2,2);
set(4,7,4,6);
set(4,7,7,12);
set(4,7,8,16);
set(4,8,4,1);
set(4,8,5,5);
set(4,8,7,9);
set(5,5,4,8);
set(5,5,5,10);
set(5,5,8,18);
set(5,6,1,3);
set(5,6,2,5);
set(5,6,7,13);
set(5,7,1,1);
set(5,7,3,5);
set(5,7,6,9);
set(5,8,3,2);
set(5,8,4,4);
set(5,8,5,6);
set(5,8,8,12);
set(6,6,3,6);
set(6,6,6,10);
set(6,6,8,16);
set(6,7,1,2);
set(6,7,2,4);
set(6,7,5,8);
set(6,7,8,14);
set(6,8,2,1);
set(6,8,3,3);
set(6,8,6,7);
set(6,8,7,9);
set(7,7,2,4);
set(7,7,4,6);
set(7,7,7,10);
set(7,8,1,1);
set(7,8,2,3);
set(7,8,4,5);
set(7,8,6,7);
set(7,8,8,11);
set(8,8,1,2);
set(8,8,3,4);
set(8,8,5,6);
set(8,8,7,8);
set(8,8,8,10);


for(i=0;i<sd;i++) {
 th[store[i][0]][store[i][1]][store[i][2]]=store[i][3];
 th[store[i][1]][store[i][0]][store[i][2]]=store[i][3];
}
};
#endif

// *************************************************** end of theta setup
int DINO=-1;

void Theta::prep(Dio& d,Mass*m,Theta*t) {
Mass*mpd=m;
pd=pdstart=t;
// set links
list[Mlinks]=ea1; list[Mlinks+1]=ea2;
list[Mlinks+2]=(ea1+PI) % (2*PI); list[Mlinks+3]=(ea2+PI) % (2*PI);
for(; mpd<=m->pdend ; pd++, mpd++) {
pd->l1=mpd->l1; pd->l2=mpd->l2; pd->l3=mpd->l3;  	// dup all exept depth=0
pd->ninst=mpd->ninst; pd->depth=0; pd->node=mpd->node;
pd->io1=(pd->node==d.list[pd->l1][0]) ? 0 : 1;
pd->io2=(pd->node==d.list[pd->l2][0]) ? 0 : 1;
pd->io3=(pd->node==d.list[pd->l3][0]) ? 0 : 1;
// set all external links inward
if(pd->l1==Mlinks+0) pd->io1=1; if(pd->l2==Mlinks+0) pd->io2=1; if(pd->l3==Mlinks+0) pd->io3=1; 
if(pd->l1==Mlinks+1) pd->io1=1; if(pd->l2==Mlinks+1) pd->io2=1; if(pd->l3==Mlinks+1) pd->io3=1;
if(pd->l1==Mlinks+2) pd->io1=1; if(pd->l2==Mlinks+2) pd->io2=1; if(pd->l3==Mlinks+2) pd->io3=1;
if(pd->l1==Mlinks+3) pd->io1=1; if(pd->l2==Mlinks+3) pd->io2=1; if(pd->l3==Mlinks+3) pd->io3=1;
pd->t1=th[*mpd->m1][*mpd->m2][*mpd->m3];
pd->t2=th[*mpd->m2][*mpd->m3][*mpd->m1];
pd->t3=th[*mpd->m3][*mpd->m1][*mpd->m2];
}; // for
init=1;
pdend=pd-1;
};

int Theta::update() {
depth++;
Angle a1,a2,a3;
Angle Pi(Angle::Pi);
a1=list[l1]; a2=list[l2]; a3=list[l3];
if(dino2==DINO) { 
p("update*********************************************update"); 
p(this-pdstart);
p(a1); p(a2); p(a3); 
p("thetas"); p(t1); p(t2); p(t3);
p("links"); p(l1); p(l2); p(l3);
p("inst"); p(ninst); }
if(io1) a1=a1+Pi;
if(io2) a2=a2+Pi;
if(io3) a3=a3+Pi;
switch(ninst) {
case 1:				// 2 posibilities: choose second else fail
if(dino2==DINO) p("case1");
switch(depth) {
	case 1:
a2=a1-t1+Pi;
a3=a1+t3+Pi;
break;
	default: if(pd==pdstart) return 2; else {pd--; return 1; };
	};
if(io2) a2=a2+Pi;		//revert to actual angle	
if(io3) a3=a3+Pi;
if(dino2==DINO) { p("setting"); p(a2); p(a3); }
list[l2]=a2; list[l3]=a3;
break;
case 2:				// fail for next
if(dino2==DINO) p("case2");
if(pd==pdstart) return 2; else {pd--; return 1; };
case 3:	
if(dino2==DINO) p("case3");
if(pd==pdstart) return 2; else {pd--; return 1; };
};	// of switch ninst
if(pd>=pdend) return 0;
pd++;
return pd->rest();
};

int Theta::rest() {
depth=0;
Angle a1,a2,a3;
Angle Pi=Angle::Pi;
a1=list[l1]; a2=list[l2]; a3=list[l3];
if(dino2==DINO) { 
p("rest*********************************************"); 
p(this-pdstart); p(a1); p(a2); p(a3); 
p("thetas"); p(t1); p(t2); p(t3);
p("links"); p(l1); p(l2); p(l3);
p("inst"); p(ninst); }
if(io1) a1=a1+Pi;
if(io2) a2=a2+Pi;
if(io3) a3=a3+Pi;
switch(ninst) {
case 1:				// 2 posibilities rest: 1st only.
				// switch(depth) { case 0:
if(dino2==DINO) p("case1");
a2=a1+t1+Pi;
a3=a1-t3+Pi;
if(io2) a2=a2+Pi;		//revert to actual angle	
if(io3) a3=a3+Pi;
if(dino2==DINO) { p("setting"); p(a2); p(a3); }
list[l2]=a2; list[l3]=a3;
break;
case 2:				// 1 posibility, check and assoc
if(dino2==DINO)  p("case2");
if(a2==a1+t1+Pi) { a3=a1-t3+Pi; } else {
	if(a2==a1-t1+Pi) { a3=a1+t3+Pi; } else {
		if(pd==pdstart) return 2; else {pd--; return 1; };
		};
	};
if(io3) a3=a3+Pi;
if(dino2==DINO) { p("setting"); p(a3); };
list[l3]=a3;
break;
case 3:		// check only
if(!((a2==a1+t1+Pi && a3==a1-t3+Pi) ||
(a2==a1-t1+Pi && a3==a1+t3+Pi))) {
if(pd==pdstart) return 2; else {pd--; return 1; }; };
};	// of switch ninst
if(dino2==DINO) p("endofswitch");
if(pd>=pdend) return 0;
pd++;
return pd->rest();
};

int Theta::make() {
int l;  Theta*qd; int j;
if(init) { init=0; pd=pdstart; l=pd->rest();
if(l==0) {
/*
cout<<"hit-first-time\n";
for(j=0;j<=(pdend-pdstart);j++) { qd=(pdstart+j);
cout<<"node: "<<qd->node<<" links: "<<qd->l1<<","<<qd->l2<<","<<qd->l3<<
" theta: "<<list[qd->l1]<<","<<list[qd->l2]<<","<<list[qd->l3]<<
" io: "<<qd->io1<<qd->io2<<qd->io3<<
" mass: "<<m.list[qd->l1]<<","<<m.list[qd->l2]<<","<<m.list[qd->l3]<<".\n"; };
*/
return 1; }
if(l==2) return 0;
}
for(;;) {
l=pd->update();
if(l==0) { 
/*
cout<<"hit-secont-time\n";
for(j=0;j<=(pdend-pdstart);j++) { qd=(pdstart+j);
cout<<"node: "<<qd->node<<" links: "<<qd->l1<<","<<qd->l2<<","<<qd->l3<<
" theta: "<<list[qd->l1]<<","<<list[qd->l2]<<","<<list[qd->l3]<<
" io: "<<qd->io1<<qd->io2<<qd->io3<<
" mass: "<<m.list[qd->l1]<<","<<m.list[qd->l2]<<","<<m.list[qd->l3]<<".\n"; };
*/
return 1;
};
if(l==2) return 0;
};
return 0;
};

// *************************************************** end of Theta functions

class Intcheck {
public:
int node;
int lnk;
int depth;
int follow;
static int init;
static int bag[Mnodes];
static int startnode;
static Intcheck *pd,*pdstart,*pdend;
static int link[Mnodes][3];
static int nlinks[Mnodes];
static int linklst[Mlinks+1][2];
int update(),rest(),loop(),main(Theta&);
void prep(Dio& d,Mass*m,Intcheck*ic);
};

void Intcheck::prep(Dio& d,Mass*m,Intcheck*ic) {
int l;
Mass *mpdstart,*mpdend,*mpd;
mpd=mpdstart=m;
mpdend=m->pdend;
for(;mpd<=mpdend;mpd++) {
node=mpd->node;
link[node][0]=mpd->l3; link[node][1]=mpd->l2; link[node][2]=mpd->l1;
}
for(node=0;node<Mnodes;node++) nlinks[node]=3;
if(pass==1) nlinks[0]=nlinks[1]=nlinks[2]=nlinks[3]=2; // first Mass::l1 is to external nodes.
if(pass>1) { nlinks[0]=1; nlinks[1]=nlinks[2]=2; }
pd=pdstart=ic;
pdend=(pdstart+Mnodes-1);
for(l=0;l<Mlinks;l++) {
linklst[l][0]=d.list[l][0]; 
linklst[l][1]=d.list[l][1];
}
init=2;
};


// exit status: 0:suceed, 1:fail-recall posible, 2:fail-fail.
// bag[node] only cleared on update due to depth>maxdepth

int Intcheck::update() {
int nextnode,lastnode,maxdepth;
maxdepth=nlinks[node];
depth++;
if(depth>=maxdepth) {
if(pd!=pdstart) { pd--; bag[node]=0; return 1; } else return 2;
}

lnk=link[node][depth];
follow=(linklst[lnk][0]==node);
if(follow) nextnode=linklst[lnk][1]; else nextnode=linklst[lnk][0];

if(pd!=pdstart) {
lastnode=(pd-1)->node;
if(nextnode==lastnode) return 1;
if(nextnode==startnode) { pdend=pd; return 0; }
if(bag[nextnode]) return 1;
}

pd++;
pd->node=nextnode;
bag[node]=1;
return pd->rest();
};



int Intcheck::rest() {
int nextnode,lastnode;
depth=0;

lnk=link[node][depth];
follow=(linklst[lnk][0]==node);
if(follow) nextnode=linklst[lnk][1]; else nextnode=linklst[lnk][0];

if(pd!=pdstart) {
lastnode=(pd-1)->node;
if(nextnode==lastnode) return 1;
if(nextnode==startnode) { pdend=pd; return 0; }
if(bag[nextnode]) { return 1; }
}

pd++;
pd->node=nextnode;
bag[node]=1;
return pd->rest();
};

int Intcheck::loop() {
int l,i;
if(init==2) { 
init=1;
startnode=3;
}
for(;;) {
if(init==1) {
init=0;
pd=pdstart;
pd->node=startnode;
for(i=0;i<Mnodes;i++) bag[i]=0;
l=pd->rest();
} else l=pd->update();
if(l==0) return 1;
if(l==2) { startnode++; init=1; if(startnode>=Mnodes) return 0; }
}
};


int Intcheck::main(Theta& t) {
int a1,a2,a3,amin,amax,bmin,bmax; int lp;
Intcheck *lpd;
int tok=1,ok=1;

while(lp=loop() && ok) {
amin=Angle::TwoPi; amax=-Angle::TwoPi;
ok=0;

for(lpd=pdstart;lpd<=pdend && !ok;lpd++) {
a1=t.list[lpd->lnk];
if(!lpd->follow) a1=(a1+Angle::Pi) % Angle::TwoPi;
a2=(a1+(3*Angle::Pi/2)) % Angle::TwoPi;
a3=(a1+(Angle::Pi/2)) % Angle::TwoPi;
if(lpd==pdstart) {amin=a2; amax=a3; }
bmin=amin; bmax=amax;
if(a3>a2) { 		// included range a2....a3 (normal)
if(amin>a2 && amin<a3 && amax>a2 && amax<a3) { ok=1; break; }
if(amin>a2 && amin<a3) bmin=a2;
if(amax>a2 && amax<a3) bmax=a3;
} else {		// included range a2..0,0..a3 (goes through zero)
if((amin>a2 || amin<a3) && (amax>a2 || amax<a3)) { ok=1; break; }
if(amin>a2 || amin<a3) bmin=a2;
if(amax>a2 || amax<a3) bmax=a3;
}
amax=bmax; amin=bmin;
};

tok=tok && ok;
};
return ok;
};


// ************************************************** end of intcheck functions
// *************************************Global definitions for main and listout

Dio dio[Mlinks+4];
Mass mass[Mass::Mpd];
Theta theta[Mass::Mpd];
Intcheck ic[Mass::Mpd];
 Intcheck i;

int dino0=1;
int dino1=1;
//int dino2=1;
int dino3=1;
int dino11=1;

void listout() {
int c1,c2,c3;
pass=1;
d.prep(dio);
d.used[0]=d.used[1]=d.used[2]=d.used[3]=2;	// first pass
d.bd=0;		// before dio pointer to.

cout<<"%ma="<<em1<<"\n"<<flush;
cout<<"%ta="<<ea1<<"\n"<<flush;
cout<<"%mb="<<em2<<"\n"<<flush;
cout<<"%tb="<<ea2<<"\n"<<flush;
p("%start");

while(d.make()) { 
 m.prep(dio[0],mass);
 c3=0;
 while(m.make()) { 
  t.prep(dio[0],mass,theta);
   while(t.make()) {
//p(dino1); p(dino2);  cout<<d; cout<<m; cout<<t;
//p(t.make());
// cout<<d; cout<<m; cout<<t;
   i.prep(dio[0],mass,ic);
   c1=i.main(t);
   if(c1) {
   if(d.bd==0) c2=1; else c2=d.check();
    if(c2) {
     c3=1;
 //    //p(dino);
   p(nodestr);
  cout<<d;
  cout<<m;
  cout<<t;
dino0++;
     }
dino11++;
    }
   dino1++;
   } // end of while(t.make)
dino2++;
  } // end of while(m.make)
if(c3) d.gen_permed_dios();
dino3++;
 } // end if while(d.make)
};

main()
{

d.gen_perms();
// 1. ordinary diograms:
pass=1;
d.prep(dio);
d.used[0]=d.used[1]=d.used[2]=d.used[3]=2;	// first pass
d.bd=0;		// before dio pointer to.

#ifdef GROUPD6
p("%group=d6");
#endif
#ifdef GROUPE6
p("%group=e6");
#endif
#ifdef GROUPE7
p("%group=e7");
#endif
#ifdef GROUPE8
p("%group=e8");
#endif

#if ORDER==2
p("%order=2");
#include "O2POLES"
#elif ORDER==3
p("%order=3");
#include "O3POLES"
#elif ORDER==4
p("%order=4");
#include "O4POLES"
#elif ORDER==5
p("%order=5");
#include "O5POLES"
#endif

p("%finished");

/* debug information: printout number of different types of diagrams

cout<<"%ma="<<em1<<"\n"<<flush;
cout<<"%ta="<<ea1<<"\n"<<flush;
cout<<"%mb="<<em2<<"\n"<<flush;
cout<<"%tb="<<ea2<<"\n"<<flush;
listout();

p(dino0);
p(dino1);
p(dino2);
p(dino3);
p(dino11);
p(d.bd);
*/

// test ************************************************************
// *************************************************end of program
/* Angle a1,a2,a3,t1,t2,t3,Pi;
Pi=Angle::Pi;
t1=42;
t2=12;
t3=6;
a1=72;
a2=90;
a3=18;
p(a2==a1+t1+Pi);
p(a3==a1-t3+Pi);
p(a2==a1-t1+Pi);
p(a3==a1+t3+Pi);
p(!((a2==a1+t1+Pi && a3==a1-t3+Pi) ||
(a2==a1-t1+Pi && a3==a1+t3+Pi) ) );

for(;;) { }
*/
/*
// 2a. second pass:
pass=2;
d.used[0]=1; d.used[1]=d.used[2]=2;
d.bd=0;
listout();

cout<<d.bd<<"\n"<<flush;
*/


};

// ********** DEFINE OBJECTS THAT ARE ONLY DECLARED **************

int Dio::used[Mlinks];
Dio* Dio::pd;
Dio* Dio::pdstart;
Dio* Dio::pdend;
int Dio::bd;
int Dio::before[5000][Mlinks+1][2];
int Dio::permlst[25][Mnodes+1];
int Dio::init;
int Dio::list[Mlinks+1][2];	// main linklist

int Mass::c[Mmasses+1][Mmasses+1][Mmasses+1];
struct Mass::S Mass::p1[Mmasses+1][64];
int Mass::p2[Mmasses+1][Mmasses+1][Mp2d];
int Mass::p1max[Mmasses+1],Mass::p2max[Mmasses+1][Mmasses+1];
Mass* Mass::pdstart,*Mass::pdend,*Mass::pd;
int Mass::init;
int Mass::list[Mlinks+4];

//class Theta:public Massdat
Theta* Theta::pd,*Theta::pdstart,*Theta::pdend;
int Theta::store[500][4],Theta::sd; //
int Theta::th[Mmasses][Mmasses][Mmasses];
int Theta::init;
int Theta::list[Mlinks+4];

//class Intcheck {
int Intcheck::init;
int Intcheck::bag[Mnodes];
int Intcheck::startnode;
Intcheck *Intcheck::pd,*Intcheck::pdstart,*Intcheck::pdend;
int Intcheck::link[Mnodes][3];
int Intcheck::nlinks[Mnodes];
int Intcheck::linklst[Mlinks+1][2];




